# Home Power Management

## Project Description
The system will allow users to manage the smart meter devices at home. 
For a user, it will be possible to set different smart meters for the different houses they own/rent, and track the power consumption statistics over time that is collected automatically. 
They can track the power consumption of different devices in different time of the day (e.g., at night). 
They can also shut down the power to one of their houses through the system, when power is not needed. 
The power distribution company can consult the power consumption statistics over period of times of all the users registered into the system.

## Scenario
First user creates an account.
Then he creates 5 houses in a loop.
After each house creation, he updates it with some new info and creates 5 devices.
After each device creation, the device uploads 50 samples of consumption while the user gets device and its samples.
After everything is created, the user deletes each device, then each house and lastly his account.
Our scenario simulates this flow for 20 users in a short span of time.

## Team members
Team members and their responsibilities
### M1:
 - Samuel Sýkora (485084) team leader
   - Team leader, developing `Power Control`, testing `User Management`
 - Aneta Jablunková (469018)
   - developing `Notifications`, testing `Power Control`
 - Filip Timko (536297)
   - developing `User Management`, testing `Power Consumption` and `Device Management`
 - Michal Cikatricis (485669)
   - developing `Power Consumption` and `Device Management`, testing `Notifications`
### M2:
 - Samuel Sýkora (485084)
   - Team leader, developing `Power Control BL and persistence`, testing `User Management`
 - Aneta Jablunková (469018)
   - developing `Notifications BL` and `Power Consumption BL and persistence`, testing `Device Management`
 - Filip Timko (536297)
   - developing `User Management BL and persistence`, testing `Power Consumption` and `Notifications`
 - Michal Cikatricis (485669)
   - developing `Device Management BL and persistence`, testing `Power Control`

### M3:
- Samuel Sýkora (485084)
   - Team leader, developing `Power Control BL and persistence`, testing `User Management`
- Aneta Jablunková (469018)
   - developing `Power Consumption BL and persistence`, testing `Device Management`
- Filip Timko (536297)
   - developing `User Management BL and persistence`, testing `Power Consumption`
- Michal Cikatricis (485669)
   - developing `Device Management BL and persistence`, testing `Power Control`

## Services

### Developed modules
1. User Management Service ( **port 8084** ) : This service will handle user authentication and authorization, user profile management.


2. Power Consumption Service ( **port 8081** ): This service will collect power consumption data from the smart meters and store it in a database. It will also allow users to track the power consumption of different devices in different time of the day (e.g., at night). It will also handle requests from the power distribution company to consult the power consumption statistics over period of times of all the users registered into the system.


3. Power Control Service ( **port 8085** ): This service will allow the user to shut down the power to one of their houses through the system, when power is not needed. It will also let the user create and remove their houses. 


4. Device management Service ( **port 8083** ): This service will handle management of the smart meters for the houses.
 
### Additional services
5. Client ( **port 8080** ): Simple client to get auth token


6. Prometheus ( **port 9090** ): Collects metrics from developed modules


7. Grafana ( **port 3000** ): Dashboard for the metrics collected

# Diagrams

![Class Diagram](diagrams/HPM_class_diagram.jpg)

---

![UseCase Diagram](diagrams/HPM_usecase.jpg)

# How to run application
## Building the application
To build the application, you will need to have Java and Maven installed on your system. You can check if you have them installed by running the following commands:
```
java -version
mvn -version
```
Once you have Java and Maven installed, you can build the application by running the following command in the root directory of the project:
```
mvn clean install
```

## Running the application with Docker Compose
To run the application with Docker Compose, you will need to have Docker and Docker Compose installed on your system. You can check if you have them installed by running the following commands:
```
docker -v
docker-compose -v
```
Once you have Docker and Docker Compose installed, you can start the application by running the following command in the root directory of the project:
```
docker-compose up
```

Once the services have started, you should be able to access your Spring Boot application by navigating to `http://localhost:<port>` in your web browser, where `<port>` is the port number specified in your application's configuration.

To stop the application, you can press CTRL + C in the terminal window where you started Docker Compose, or run the following command:
```
docker-compose down
```

## Using the applications
As most of the routes are protected by the oauth, first you need to visit the client at ```localhost:8080```.
After logging in, there is a show auth token link above the logout button. This lets you copy your auth token to use
with either swagger or the scenario.

## Grafana
The login information are ***username***: ```admin```, ***password***: ```admin```.
After logging in, navigate to Dashboards -> General -> Home power management

## How to run the test scenario
### Requirements
- Java
- Maven
- JMeter(https://jmeter.apache.org/download_jmeter.cgi)
- docker + docker compose
### How to actually run
First you need to get the servers up and running, either by running ```mvn spring-boot:run``` in all of the services,
or by running ```docker-compose up --build``` in the root(home-power-management) directory.
Then you can move to actually running the jmeter. Either in headless mode like this:
```jmeter -JauthToken=your_auth_token -n -t your_test_plan.jmx -l your_test_results.jtl```
Where:
- ```jmeter``` should be the installed binary
- ```your_auth_token``` should be retireved via client (on ```localhost:8080```)
- ```your_test_plan.jmx``` should be either relative, or absolute path to the file (something like: ```testplan/Home-power-management.jmx```)
- ```your_test_results.jtl``` should be the path to the file that will hold your results (something like: ```home-power-management-results.jtl```)

Alternatively you can run it in gui mode like this:
```jmeter -JauthToken=your_auth_token```
Which allows you to see nice summaries and metrics
