package cz.muni.pa165.client;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClient;
import org.springframework.security.oauth2.client.annotation.RegisteredOAuth2AuthorizedClient;
import org.springframework.security.oauth2.core.OAuth2AccessToken;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;

import java.net.ConnectException;
import java.time.ZoneId;
import java.util.TreeSet;

@Controller
public class MainController {
    private static final Logger log = LoggerFactory.getLogger(MainController.class);

    /**
     * Home page accessible even to non-authenticated users. Displays user personal data.
     */
    @GetMapping("/")
    public String index(Model model, @AuthenticationPrincipal OidcUser user) {
        log.debug("********************************************************");
        log.debug("* index() called                                       *");
        log.debug("********************************************************");
        log.debug("user {}", user == null ? "is anonymous" : user.getSubject());

        // put obtained user data into a model attribute named "user"
        model.addAttribute("user", user);

        // put issuer name into a model attribute named "issuerName"
        if (user != null) {
            model.addAttribute("issuerName",
                    "https://oidc.muni.cz/oidc/".equals(user.getIssuer().toString()) ? "MUNI" : "Google");
            model.addAttribute("myAuthToken", user.getAccessTokenHash());
        }

        // return the name of a Thymeleaf HTML template that
        // will be searched in src/main/resources/templates with .html suffix
        return "index";
    }

    @GetMapping("/myToken")
    public String myToken(Model model, @AuthenticationPrincipal OidcUser user, @RegisteredOAuth2AuthorizedClient OAuth2AuthorizedClient oauth2Client) {

        OAuth2AccessToken accessToken = oauth2Client.getAccessToken();
        // log access token
        log.debug("access token principal: {}", oauth2Client.getPrincipalName());
        log.debug("access token scopes: {}", accessToken.getScopes());
        log.debug("access token issued: {}", accessToken.getIssuedAt().atZone(ZoneId.systemDefault()));
        log.debug("access token expires: {}", accessToken.getExpiresAt().atZone(ZoneId.systemDefault()));
        log.debug("access token value: {}", accessToken.getTokenValue());

        // for display
        model.addAttribute("user", user);
        model.addAttribute("scopes", new TreeSet<>(accessToken.getScopes()));
        model.addAttribute("myAccessToken", accessToken.getTokenValue());

        return "mytoken";
    }
}
