package cz.muni.pa165.generated.device_management;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.FileCopyUtils;
import org.springframework.core.env.Environment;


import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

@Component
@Order(1)
public class DatabaseSeeder implements CommandLineRunner {

    private final JdbcTemplate jdbcTemplate;

    @Autowired
    private Environment environment;

    public DatabaseSeeder(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }
    @Override
    public void run(String... args) {
        if (shouldClearDatabase()) {
            clearDatabase();
        }

        if (shouldSeedDatabase()) {
            seedDatabase();
        }
    }

    private boolean shouldSeedDatabase() {
        return environment.getProperty("app.seed", Boolean.class, false);
    }

    private boolean shouldClearDatabase() {
        return environment.getProperty("app.clear", Boolean.class, false);
    }

    private void seedDatabase() {
        executeSqlFile("seed.sql");
    }

    private void clearDatabase() {
        executeSqlFile("clear.sql");
    }

    private void executeSqlFile(String filePath) {
        Resource resource = new ClassPathResource(filePath);
        try (Reader reader = new InputStreamReader(resource.getInputStream(), StandardCharsets.UTF_8)) {
            String sql = FileCopyUtils.copyToString(reader);
            Arrays.stream(sql.split(";"))
                    .map(String::trim)
                    .filter(query -> !query.isEmpty())
                    .forEach(jdbcTemplate::execute);
        } catch (IOException e) {
            throw new RuntimeException("Failed to read SQL file: " + filePath, e);
        }
    }
}
