package cz.muni.pa165.generated.device_management.Service;

import cz.muni.pa165.generated.device_management.data.model.Device;
import cz.muni.pa165.generated.device_management.data.repository.DeviceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.webjars.NotFoundException;

import java.util.List;

@Service
public class DeviceService {

    private final DeviceRepository repository;

    @Autowired
    public DeviceService(DeviceRepository repository) {
        this.repository = repository;
    }

    public Device createDevice(Device device) {
        if (repository.findByMAC(device.getMacAddress()).isPresent()) {
            throw new NotFoundException("MAC address " + device.getMacAddress() + " is not unique.");
        }
        return repository.save(device);
    }

    public Device updateDevice(String id, Device device) {
        var fetchedDevice = repository.findById(id);
        if (fetchedDevice.isEmpty()) {
            throw new NotFoundException("Device with ID " + id + " does not exist.");
        }
        return repository.save(device);
    }

    public void deleteDevice(String deviceId) {
        if (!repository.existsById(deviceId)) {
            throw new NotFoundException("Device with id " + deviceId + " does not exist.");
        }
        repository.deleteById(deviceId);
    }

    public Device getDevice(String deviceId) {
        if (!repository.existsById(deviceId)) {
            throw new NotFoundException("Device with ID " + deviceId + " does not exist.");
        }
        return repository.findById(deviceId).get();
    }

    public List<Device> getDevicesByHouseId(String houseId) {
        var devices = repository.findByHouseId(houseId);
        if (devices.isEmpty()) {
            throw new NotFoundException("No devices found for house " + houseId);
        }
        return devices;
    }

    public Device getLastCreatedDeviceByHouseId(String houseId) {
        Pageable pageable = PageRequest.of(0, 1);
        List<Device> devices = repository.findByHouseIdSorted(houseId, pageable);
        if (devices.isEmpty()) {
            throw new NotFoundException("No devices created for house " + houseId);
        }
        return devices.get(0);
    }
}
