package cz.muni.pa165.generated.device_management;

import cz.muni.pa165.generated.device_management.Service.DeviceService;
import cz.muni.pa165.generated.device_management.data.model.Device;
import cz.muni.pa165.generated.device_management.data.repository.DeviceRepository;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.webjars.NotFoundException;

import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class DeviceManagementServiceTest {

    @InjectMocks
    private DeviceService deviceService;

    @Mock
    private DeviceRepository deviceRepository;

    @Test
    public void createDevice() {
        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("testName");
        device.setHouseId("testHouseId");

        when(deviceRepository.save(any(Device.class))).thenReturn(device);
        var createdDevice = deviceService.createDevice(device);

        assertEquals(device.getMacAddress(), createdDevice.getMacAddress());
        assertEquals(device.getName(), createdDevice.getName());
        assertEquals(device.getHouseId(), createdDevice.getHouseId());
    }

    @Test
    public void updateNonExistenceDevice() {
        var deviceId = "deviceId";
        var newDevice = new Device();
        newDevice.setMacAddress("testMacAddress");
        newDevice.setName("testName");
        newDevice.setHouseId("testHouseId");

        when(deviceRepository.findById(deviceId)).thenReturn(Optional.empty());

        assertThrows(NotFoundException.class, () -> deviceService.updateDevice(deviceId, newDevice));

        verify(deviceRepository, times(1)).findById(deviceId);
        verify(deviceRepository, never()).save(any(Device.class));
    }

    @Test
    public void testUpdateDeviceMacAddress() {
        var deviceId = "deviceId";
        var oldDevice = new Device();
        oldDevice.setHouseId("testHouseId");
        oldDevice.setMacAddress("oldMacAddress");
        oldDevice.setName("testName");

        var newDevice = new Device();
        newDevice.setMacAddress("testMacAddress");
        newDevice.setName("testName");
        newDevice.setHouseId("testHouseId");

        when(deviceRepository.findById(deviceId)).thenReturn(Optional.of(oldDevice));
        when(deviceRepository.save(any(Device.class))).thenReturn(newDevice);

        var updatedDevice = deviceService.updateDevice(deviceId, newDevice);

        verify(deviceRepository, times(1)).findById(deviceId);
        verify(deviceRepository, times(1)).save(any(Device.class));

        assertEquals(newDevice.getMacAddress(), updatedDevice.getMacAddress());
        assertEquals(newDevice.getName(), updatedDevice.getName());
        assertEquals(newDevice.getHouseId(), updatedDevice.getHouseId());
    }

    @Test
    public void testUpdateDeviceName() {
        var deviceId = "deviceId";
        var oldDevice = new Device();
        oldDevice.setMacAddress("testMacAddress");
        oldDevice.setName("testName");
        oldDevice.setHouseId("testHouseId");

        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("newName");
        device.setHouseId("testHouseId");

        when(deviceRepository.findById(deviceId)).thenReturn(Optional.of(oldDevice));
        when(deviceRepository.save(any(Device.class))).thenReturn(device);

        var updatedDevice = deviceService.updateDevice(deviceId, device);

        verify(deviceRepository, times(1)).findById(deviceId);
        verify(deviceRepository, times(1)).save(any(Device.class));

        assertEquals(device.getMacAddress(), updatedDevice.getMacAddress());
        assertEquals(device.getName(), updatedDevice.getName());
        assertEquals(device.getHouseId(), updatedDevice.getHouseId());
    }

    @Test
    public void testUpdateDeviceHouseId() {
        var deviceId = "deviceId";
        var oldDevice = new Device();
        oldDevice.setMacAddress("testMacAddress");
        oldDevice.setName("testName");
        oldDevice.setHouseId("testHouseId");

        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("testName");
        device.setHouseId("newHouseId");

        when(deviceRepository.findById(deviceId)).thenReturn(Optional.of(oldDevice));
        when(deviceRepository.save(any(Device.class))).thenReturn(device);

        var updatedDevice = deviceService.updateDevice(deviceId, device);

        verify(deviceRepository, times(1)).findById(deviceId);
        verify(deviceRepository, times(1)).save(any(Device.class));

        assertEquals(device.getMacAddress(), updatedDevice.getMacAddress());
        assertEquals(device.getName(), updatedDevice.getName());
        assertEquals(device.getHouseId(), updatedDevice.getHouseId());
    }

    @Test
    public void testUpdateDeviceAll() {
        var deviceId = "deviceId";
        var oldDevice = new Device();
        oldDevice.setMacAddress("testMacAddress");
        oldDevice.setName("testName");
        oldDevice.setHouseId("testHouseId");

        var device = new Device();
        device.setMacAddress("newMacAddress");
        device.setName("newName");
        device.setHouseId("newHouseId");

        when(deviceRepository.findById(deviceId)).thenReturn(Optional.of(oldDevice));
        when(deviceRepository.save(any(Device.class))).thenReturn(device);

        var updatedDevice = deviceService.updateDevice(deviceId, device);

        verify(deviceRepository, times(1)).findById(deviceId);
        verify(deviceRepository, times(1)).save(any(Device.class));

        assertEquals(device.getMacAddress(), updatedDevice.getMacAddress());
        assertEquals(device.getName(), updatedDevice.getName());
        assertEquals(device.getHouseId(), updatedDevice.getHouseId());
    }

    @Test
    public void deleteNonExistingDevice() {
        String nonExistingDeviceId = "non-existing-device-id";
        when(deviceRepository.existsById(nonExistingDeviceId)).thenReturn(false);
        
        assertThrows(NotFoundException.class, () -> deviceService.deleteDevice(nonExistingDeviceId));

        verify(deviceRepository, times(1)).existsById(nonExistingDeviceId);
        verify(deviceRepository, never()).deleteById(nonExistingDeviceId);
    }

    @Test
    public void deleteDevice() {
        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("testName");
        device.setHouseId("testHouseId");

        when(deviceRepository.save(any(Device.class))).thenReturn(device);
        var savedDevice = deviceRepository.save(device);

        String existingDeviceId = savedDevice.getId();
        when(deviceRepository.existsById(existingDeviceId)).thenReturn(true);

        deviceService.deleteDevice(existingDeviceId);

        verify(deviceRepository, times(1)).existsById(existingDeviceId);
        verify(deviceRepository, times(1)).deleteById(existingDeviceId);
    }

    @Test
    public void getNonExistingDevice() {
        String nonExistingDeviceId = "non-existing-device-id";
        when(deviceRepository.existsById(nonExistingDeviceId)).thenReturn(false);

        assertThrows(NotFoundException.class, () -> deviceService.getDevice(nonExistingDeviceId));

        verify(deviceRepository, times(1)).existsById(nonExistingDeviceId);
        verify(deviceRepository, never()).findById(nonExistingDeviceId);

    }

    @Test
    public void getDevice() {
        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("testName");
        device.setHouseId("testHouseId");

        when(deviceRepository.save(any(Device.class))).thenReturn(device);
        var savedDevice = deviceRepository.save(device);

        String existingDeviceId = savedDevice.getId();
        when(deviceRepository.existsById(existingDeviceId)).thenReturn(true);
        when(deviceRepository.findById(existingDeviceId)).thenReturn(Optional.of(savedDevice));

        var responseDevice = deviceService.getDevice(existingDeviceId);

        verify(deviceRepository, times(1)).existsById(existingDeviceId);
        verify(deviceRepository, times(1)).findById(existingDeviceId);
        assertEquals(device, responseDevice);
    }

    @Test
    public void getDevicesByHouseId() {
        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("testName");
        device.setHouseId("testHouseId");

        when(deviceRepository.save(any(Device.class))).thenReturn(device);
        var savedDevice = deviceRepository.save(device);

        var houseId = savedDevice.getHouseId();
        List<Device> getDevices = List.of(savedDevice);
        when(deviceRepository.findByHouseId(houseId)).thenReturn(getDevices);

        var responseDevice = deviceService.getDevicesByHouseId(houseId);

        verify(deviceRepository, times(1)).findByHouseId(houseId);
        assertEquals(getDevices, responseDevice);
    }

    @Test
    public void testGetLastCreatedDeviceByHouseId_NoDevices() {
        String houseId = "houseId";
        Pageable pageable = PageRequest.of(0, 1);

        when(deviceRepository.findByHouseIdSorted(houseId, pageable)).thenReturn(List.of());

        assertThrows(NotFoundException.class, () -> {
            deviceService.getLastCreatedDeviceByHouseId(houseId);
        });
    }

    @Test
    public void testGetLastCreatedDeviceByHouseId() {
        Pageable pageable = PageRequest.of(0, 1);
        var device = new Device();
        device.setMacAddress("testMacAddress");
        device.setName("testName");
        device.setHouseId("testHouseId");

        when(deviceRepository.save(any(Device.class))).thenReturn(device);
        deviceRepository.save(device);
        var lastSavedDevice = deviceRepository.save(device);

        var houseId = lastSavedDevice.getHouseId();
        List<Device> allDevices = List.of(lastSavedDevice);
        when(deviceRepository.findByHouseIdSorted(houseId, pageable)).thenReturn(allDevices);

        var actualOutput = deviceService.getLastCreatedDeviceByHouseId(houseId);
        assertEquals(lastSavedDevice, actualOutput);
    }

}
