package cz.muni.pa165.generated.power_consumption;

import com.github.javafaker.Faker;
import cz.muni.pa165.generated.power_consumption.data.model.DeviceConsumption;
import cz.muni.pa165.generated.power_consumption.data.repository.DeviceConsumptionRepository;
import cz.muni.pa165.generated.power_consumption.model.ConsumptionUnit;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.OffsetDateTime;
import java.util.List;
import java.util.Random;

@Component
public class DataSeeder implements CommandLineRunner {
    Logger log = LoggerFactory.getLogger(DataSeeder.class);

    private final DeviceConsumptionRepository deviceConsumptionRepository;

    private final PasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public DataSeeder(DeviceConsumptionRepository deviceConsumptionRepository) {
        this.deviceConsumptionRepository = deviceConsumptionRepository;
    }

    @Override
    public void run(String... args) throws Exception {
        if (deviceConsumptionRepository.count() == 0) {
            log.info("Seeding data");
            seedData();
        }
    }

    private void seedData() {
        for (int i = 0; i < 50; i++) {
            Faker faker = new Faker(new Random(i));
            String macAddress = faker.internet().macAddress();
            BigDecimal consumption = BigDecimal.valueOf(faker.number().randomDouble(2, 0, 100));
            OffsetDateTime dateTime = faker.date().past(30, java.util.concurrent.TimeUnit.DAYS).toInstant().atOffset(java.time.ZoneOffset.UTC);

            DeviceConsumption deviceConsumption = new DeviceConsumption();
            deviceConsumption.setDeviceMacAddress(macAddress);
            deviceConsumption.setConsumption(consumption);
            deviceConsumption.setUnit(ConsumptionUnit.KW_H);
            deviceConsumption.setDateTime(dateTime);

            deviceConsumptionRepository.save(deviceConsumption);
        }
    }
}
