package cz.muni.pa165.generated.power_consumption.Service;

import cz.muni.pa165.generated.power_consumption.data.model.DeviceConsumption;
import org.hibernate.ObjectNotFoundException;

import java.time.OffsetDateTime;
import java.util.List;

/**
 * The interface {@code PowerConsumptionService} provides methods for managing power consumption data for devices.
 */
public interface PowerConsumptionService {
    /**
     * Returns the consumption data with specified ID.
     *
     * @param id the ID of the consumption data
     * @return the {@code DeviceConsumption} object containing the consumption data
     * @throws ObjectNotFoundException if the consumption data with the specified ID does not exist
     */
    DeviceConsumption getConsumption(String id) throws ObjectNotFoundException;

    /**
     * Returns a list of all consumption data for all devices.
     *
     * @return a {@code List} of {@code DeviceConsumption} objects containing all the consumption data
     */
    List<DeviceConsumption> listAllConsumptions();

    /**
     * Returns the consumption data for the device with the specified ID.
     *
     * @param deviceId the ID of the device to retrieve the consumption data for
     * @return a {@code List} of {@code DeviceConsumption} objects containing the consumption data for the specified device
     */
    List<DeviceConsumption> listAllConsumptionsWithSpecificDevice(String deviceId);

    /**
     * Returns a list of all devices for which consumption data is available.
     *
     * @return a {@code List} of device IDs
     */
    List<String> listAllDevices();

    /**
     * Creates a new {@code DeviceConsumption} object for the specified device with the specified unit and date/time.
     *
     * @param deviceConsumption the new {@code DeviceConsumption} object
     * @return the newly created {@code DeviceConsumption} object
     */
    DeviceConsumption createDeviceConsumption(DeviceConsumption deviceConsumption);

    /**
     * Updates the consumption data for the device with the specified ID.
     *
     * @param id the ID of the consumption data to update
     * @param deviceConsumption the {@code DeviceConsumption} object containing the updated information
     * @return the updated {@code DeviceConsumption} object
     * @throws ObjectNotFoundException if the consumption data with the specified ID does not exist
     */
    void updateDeviceConsumption(String id, DeviceConsumption deviceConsumption) throws ObjectNotFoundException;

    /**
     * Deletes the consumption data with the specified ID.
     *
     * @param id the ID of the consumption data to delete
     * @throws ObjectNotFoundException if the consumption data with the specified ID does not exist
     */
    void deleteDeviceConsumption(String id) throws ObjectNotFoundException;

    /**
     * Returns a list of all consumption data for the device with the specified ID, between the specified start and end dates/times.
     *
     * @param deviceId the ID of the device to retrieve the consumption data for
     * @param startDate the start date/time of the period to retrieve the consumption data for
     * @param endDate the end date/time of the period to retrieve the consumption data for
     * @return a {@code List} of {@code DeviceConsumption} objects containing the consumption data for the device during the specified period
     */
    List<DeviceConsumption> getPowerConsumption(String deviceId, OffsetDateTime startDate, OffsetDateTime endDate);
}
