package cz.muni.pa165.generated.power_consumption.Service;

import cz.muni.pa165.generated.power_consumption.data.repository.DeviceConsumptionRepository;
import cz.muni.pa165.generated.power_consumption.data.model.DeviceConsumption;
import jakarta.transaction.Transactional;
import org.hibernate.ObjectNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.OffsetDateTime;
import java.util.List;

@Service
public class PowerConsumptionServiceImpl implements PowerConsumptionService {

    private final DeviceConsumptionRepository deviceConsumptionRepository;

    @Autowired
    public PowerConsumptionServiceImpl(DeviceConsumptionRepository deviceConsumptionRepository) {
        this.deviceConsumptionRepository = deviceConsumptionRepository;
    }

    @Override
    public DeviceConsumption getConsumption(String id) throws ObjectNotFoundException {
        return deviceConsumptionRepository.findById(id)
                .orElseThrow(() -> new ObjectNotFoundException("Power consumption not found", new Throwable()));
    }

    @Override
    public List<DeviceConsumption> listAllConsumptions() {
        return deviceConsumptionRepository.findAll();
    }

    @Override
    public List<DeviceConsumption> listAllConsumptionsWithSpecificDevice(String deviceId) {
        return deviceConsumptionRepository.findByDeviceId(deviceId);
    }

    @Override
    public List<String> listAllDevices() {
        return deviceConsumptionRepository.findAllDevices();
    }

    @Override
    public DeviceConsumption createDeviceConsumption(DeviceConsumption deviceConsumption) {
        return deviceConsumptionRepository.save(deviceConsumption);
    }

    @Transactional
    @Override
    public void updateDeviceConsumption(String id, DeviceConsumption deviceConsumption) throws ObjectNotFoundException {
        getConsumption(id);
        deviceConsumptionRepository.update(
                id,
                deviceConsumption.getDeviceMacAddress(),
                deviceConsumption.getConsumption(),
                deviceConsumption.getUnit(),
                deviceConsumption.getDateTime());
    }

    @Override
    public void deleteDeviceConsumption(String id) throws ObjectNotFoundException {
        getConsumption(id);
        deviceConsumptionRepository.deleteById(id);
    }

    @Override
    public List<DeviceConsumption> getPowerConsumption(String deviceId, OffsetDateTime startDate, OffsetDateTime endDate) {
        return deviceConsumptionRepository.findInInterval(deviceId, startDate, endDate);
    }
}
