package cz.muni.pa165.generated.power_consumption;

import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.matchesRegex;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
class PowerConsumptionServerTest {
    private static final Logger log = LoggerFactory.getLogger(PowerConsumptionServer.class);

    @Autowired
    private MockMvc mockMvc;

    @Test
    void getPowerConsumptions() throws Exception {
        log.debug("getPowerConsumption() running");
        String response = mockMvc.perform(get("/api/powerConsumption"))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();

        assertThat(response, matchesRegex("\\[.*]"));
    }

    void deletePowerConsumption(String id) throws Exception {
        log.debug("deletePowerConsumption() running");
        String response = mockMvc.perform(delete("/api/powerConsumption/" + id))
                .andExpect(status().isNoContent())
                .andReturn().getResponse().getContentAsString();

        assertThat(response, is(""));
    }

    @Test
    void testDelete() throws Exception {
        String response = mockMvc.perform(get("/api/powerConsumption"))
                .andReturn().getResponse().getContentAsString();
        String id = response.split(":")[1].replace("\"", "").split(",")[0];
        deletePowerConsumption(id);
    }

    @Test
    void deletePowerConsumptionNonExistent() throws Exception {
        String response = mockMvc.perform(delete("/api/powerConsumption/id"))
                .andExpect(status().isNotFound())
                .andReturn().getResponse().getContentAsString();
        assertThat(response, is(""));
    }

    @Test
    void postPowerConsumption() throws Exception {
        mockMvc.perform(post("/api/powerConsumption")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("""
                            {
                              "deviceMacAddress": "address",
                              "consumption": 0,
                              "unit": "kw/h",
                              "dateTime": "2023-04-16T20:13:45.281Z"
                            }
                        """)
                )
                .andExpect(status().isCreated())
                .andReturn();
    }

    @Test
    void postSample() throws Exception {
        mockMvc.perform(post("/api/sample")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("""
                            {
                                "deviceMacAddress": "address",
                                "consumption": 0,
                                "unit": "kw/h"
                            }
                """)
        )
                .andExpect(status().isCreated())
                .andReturn();
    }
}