package cz.muni.pa165.generated.power_consumption.data.repository;

import cz.muni.pa165.generated.power_consumption.data.model.DeviceConsumption;
import cz.muni.pa165.generated.power_consumption.model.ConsumptionUnit;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;

import java.math.BigDecimal;
import java.time.OffsetDateTime;
import java.util.Arrays;
import java.util.List;

@SpringBootTest
class DeviceConsumptionRepositoryTest {
    @Autowired
    private DeviceConsumptionRepository deviceConsumptionRepository;

    DeviceConsumption dc;

    @BeforeEach
    public void init() {
        deviceConsumptionRepository.deleteAll();
        dc = new DeviceConsumption();
        dc.setDeviceMacAddress("address");
        dc.setUnit(ConsumptionUnit.KW_H);
        dc.setDateTime(OffsetDateTime.now());
        deviceConsumptionRepository.save(dc);
    }

    @AfterEach
    public void clean() {
        deviceConsumptionRepository.deleteAll();
    }

    @Test
    void findByDeviceId() {
        List<DeviceConsumption> devices = deviceConsumptionRepository.findByDeviceId("address");
        assertThat(devices.size(), is(1));
        DeviceConsumption c = devices.get(0);

        assertThat(c.getId(), is(dc.getId()));
        assertThat(c.getDeviceMacAddress(), is(dc.getDeviceMacAddress()));
        assertThat(c.getUnit(), is(dc.getUnit()));
    }

    @Test
    void findAllDevices() {
        List<String> addresses = deviceConsumptionRepository.findAllDevices();
        assertThat(addresses.size(), is(1));

        assertThat(addresses.get(0), is(dc.getDeviceMacAddress()));
    }

    @Test
    void update() {
        DeviceConsumption c = deviceConsumptionRepository.findById(dc.getId()).get();
        assertThat(c.getDeviceMacAddress(), is("address"));
    }

    @Test
    void findInInterval() {
        List<DeviceConsumption> devices = deviceConsumptionRepository.findInInterval(dc.getId(), dc.getDateTime().minusDays(2), dc.getDateTime().minusDays(1));
        assertThat(devices.size(), is(0));
    }
}