package cz.muni.pa165.generated.power_control.facade;

import cz.muni.pa165.generated.power_control.model.HouseResponse;
import cz.muni.pa165.generated.power_control.model.HouseResponseList;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;

/**
 * Interface for the House Facade. This facade provides a simplified interface to the HouseService, presenting methods to manage House objects.
 */
public interface HouseFacade {

    /**
     * Switches the power of a house and returns the updated status.
     *
     * @param id the id of the house
     * @return the updated house response object
     */
    HouseResponse switchPower(String id);

    /**
     * Retrieves a list of house responses owned by a specific owner.
     *
     * @param id the id of the owner
     * @return the list of house responses owned by the owner
     */
    HouseResponseList getAllByOwnerId(String id);

    /**
     * Creates a new house and returns the created house response.
     *
     * @param house the details of the new house to be created
     * @return the created house response object
     */
    HouseResponse create(NewHouse house);

    /**
     * Retrieves a house response by its id.
     *
     * @param id the id of the house
     * @return the house response object with the provided id
     */
    HouseResponse get(String id);

    /**
     * Updates the details of a house and returns the updated house response.
     *
     * @param id the id of the house to be updated
     * @param house the details to be updated in the house
     * @return the updated house response object
     */
    HouseResponse update(String id, UpdateHouse house);

    /**
     * Deletes a house by its id.
     *
     * @param id the id of the house to be deleted
     */
    void delete(String id);
}
