package cz.muni.pa165.generated.power_control.service;

import cz.muni.pa165.generated.power_control.data.model.House;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;

import java.util.List;

/**
 * Interface for House Service. This service is used to manage House objects in the application.
 */
public interface HouseService {

    /**
     * Switches the power of a house.
     *
     * @param id the id of the house
     * @return the updated house object
     */
    House switchHousePower(String id);

    /**
     * Retrieves a list of houses owned by a specific owner.
     *
     * @param id the id of the owner
     * @return the list of houses owned by the owner
     */
    List<House> getHousesByOwnerId(String id);

    /**
     * Creates a new house.
     *
     * @param newHouse the details of the new house to be created
     * @return the created house object
     */
    House createHouse(NewHouse newHouse);

    /**
     * Retrieves a house by its id.
     *
     * @param id the id of the house
     * @return the house object with the provided id
     */
    House getHouseById(String id);

    /**
     * Updates the details of a house.
     *
     * @param id the id of the house to be updated
     * @param house the details to be updated in the house
     * @return the updated house object
     */
    House updateHouse(String id, UpdateHouse house);

    /**
     * Deletes a house by its id.
     *
     * @param id the id of the house to be deleted
     */
    void deleteHouse(String id);
}
