package cz.muni.pa165.generated.power_control.service;

import cz.muni.pa165.generated.power_control.data.model.House;
import cz.muni.pa165.generated.power_control.data.repository.HouseRepository;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.webjars.NotFoundException;

import java.util.List;

@Service
public class HouseServiceImpl implements HouseService{
    private final HouseRepository repository;

    @Autowired
    public HouseServiceImpl(HouseRepository repository) {
        this.repository = repository;
    }

    @Override
    @Transactional
    public House switchHousePower(String id) {
        var house = repository.getReferenceById(id);
        house.setEnabled(!house.isEnabled());
        return repository.saveAndFlush(house);
    }

    @Override
    public List<House> getHousesByOwnerId(String id) {
        return repository.findHousesByOwnerId(id);
    }

    @Override
    public House createHouse(NewHouse newHouse) {
        var house = new House()
                .setName(newHouse.getName())
                .setAddress(newHouse.getAddress())
                .setOwnerId(newHouse.getOwnerId())
                .setEnabled(true);
        return repository.saveAndFlush(house);
    }

    @Override
    public House getHouseById(String id) {
        return repository.getReferenceById(id);
    }

    @Override
    @Transactional
    public House updateHouse(String id, UpdateHouse house) {
        var foundHouse = repository.findById(id);
        if (foundHouse.isEmpty()) {
            throw new NotFoundException("House with id " + id +" not found");
        }
        var nameToUpdate = house.getName();
        var addressToUpdate = house.getAddress();
        if (nameToUpdate.isPresent()) {
            foundHouse.get().setName(nameToUpdate.get());
        }
        if (addressToUpdate.isPresent()) {
            foundHouse.get().setAddress(addressToUpdate.get());
        }
        return repository.saveAndFlush(foundHouse.get());
    }

    @Override
    public void deleteHouse(String id) {
        if (repository.findById(id).isEmpty()) {
            throw new NotFoundException("House with id " + id + " not found");
        }
        repository.deleteById(id);
    }
}
