package cz.muni.pa165.generated.power_control;

import cz.muni.pa165.generated.power_control.data.model.House;
import cz.muni.pa165.generated.power_control.facade.HouseFacadeImpl;
import cz.muni.pa165.generated.power_control.mappers.HouseMapper;
import cz.muni.pa165.generated.power_control.model.HouseResponse;
import cz.muni.pa165.generated.power_control.model.HouseResponseList;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;
import cz.muni.pa165.generated.power_control.service.HouseService;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class HouseFacadeTests {

    @InjectMocks
    private HouseFacadeImpl houseFacadeImpl;

    @Mock
    private HouseService houseService;

    @Mock
    private HouseMapper houseMapper;

    @Test
    public void testSwitchPower() {
        var id = "houseId";
        var house = new House()
                .setEnabled(true);
        var houseResponse = new HouseResponse()
                .enabled(true);

        when(houseService.switchHousePower(id)).thenReturn(house);
        when(houseMapper.mapToDto(house)).thenReturn(houseResponse);

        HouseResponse result = houseFacadeImpl.switchPower(id);

        assertEquals(houseResponse, result);
        verify(houseService, times(1)).switchHousePower(id);
        verify(houseMapper, times(1)).mapToDto(house);
    }

    @Test
    public void testGetHousesByOwnerNotEmpty() {
        var owner = "owner";
        List<House> houseList = new ArrayList<>();
        houseList.add(new House()
                .setOwnerId(owner));
        var houseResponseList = new HouseResponseList().addHousesItem(
                new HouseResponse().ownerId(owner)
        );

        when(houseService.getHousesByOwnerId(owner)).thenReturn(houseList);
        when(houseMapper.mapToList(houseList)).thenReturn(houseResponseList);

        HouseResponseList result = houseFacadeImpl.getAllByOwnerId(owner);

        assertEquals(houseResponseList, result);
        verify(houseService, times(1)).getHousesByOwnerId(owner);
        verify(houseMapper, times(1)).mapToList(houseList);
    }

    @Test
    public void testGetHousesByOwnerEmpty() {
        var owner = "owner";
        List<House> houseList = new ArrayList<>();
        var houseResponseList = new HouseResponseList();

        when(houseService.getHousesByOwnerId(owner)).thenReturn(houseList);
        when(houseMapper.mapToList(houseList)).thenReturn(houseResponseList);

        HouseResponseList result = houseFacadeImpl.getAllByOwnerId(owner);

        assertEquals(houseResponseList, result);
        verify(houseService, times(1)).getHousesByOwnerId(owner);
        verify(houseMapper, times(1)).mapToList(houseList);
    }

    @Test
    public void testCreate() {
        var newHouse = new NewHouse()
                .name("name")
                .address("address")
                .ownerId("owner");
        var house = new House()
                .setName("name")
                .setAddress("address")
                .setOwnerId("owner")
                .setId("houseId")
                .setEnabled(false);
        var houseResponse = new HouseResponse()
                .name("name")
                .address("address")
                .ownerId("owner")
                .id("houseId")
                .enabled(false);

        when(houseService.createHouse(newHouse)).thenReturn(house);
        when(houseMapper.mapToDto(house)).thenReturn(houseResponse);

        HouseResponse result = houseFacadeImpl.create(newHouse);

        assertEquals(houseResponse, result);
        verify(houseService, times(1)).createHouse(newHouse);
        verify(houseMapper, times(1)).mapToDto(house);
    }

    @Test
    public void testGet() {
        var houseId = "houseId";
        var house = new House()
                .setId(houseId);
        var houseResponse = new HouseResponse()
                .id(houseId);

        when(houseService.getHouseById(houseId)).thenReturn(house);
        when(houseMapper.mapToDto(house)).thenReturn(houseResponse);

        HouseResponse result = houseFacadeImpl.get(houseId);

        assertEquals(houseResponse, result);
        verify(houseService, times(1)).getHouseById(houseId);
        verify(houseMapper, times(1)).mapToDto(house);
    }

    @Test
    public void testUpdate() {
        var houseId = "houseId";
        var updateHouse = new UpdateHouse()
                .name("house")
                .address("address");
        var house = new House()
                .setName("house")
                .setAddress("address")
                .setId(houseId);
        var houseResponse = new HouseResponse()
                .name("house")
                .address("address")
                .id(houseId);

        System.out.println(house.toString());
        when(houseService.updateHouse(houseId, updateHouse)).thenReturn(house);
        when(houseMapper.mapToDto(house)).thenReturn(houseResponse);

        HouseResponse result = houseFacadeImpl.update(houseId, updateHouse);

        assertEquals(houseResponse, result);
        verify(houseService, times(1)).updateHouse(houseId, updateHouse);
        verify(houseMapper, times(1)).mapToDto(house);
    }

    @Test
    public void testDelete() {
        var houseId = "houseId";

        houseService.deleteHouse(houseId);

        verify(houseService, times(1)).deleteHouse(houseId);
    }

}
