package cz.muni.pa165.generated.power_control;

import cz.muni.pa165.generated.power_control.data.model.House;
import cz.muni.pa165.generated.power_control.data.repository.HouseRepository;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;
import cz.muni.pa165.generated.power_control.service.HouseServiceImpl;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class HouseServiceTests {

    @InjectMocks
    private HouseServiceImpl houseService;

    @Mock
    private HouseRepository houseRepository;

    @Test
    public void testSwitchHousePower() {
        var houseId = "houseId";
        var houseIn = new House()
                .setEnabled(false);

        when(houseRepository.getReferenceById(houseId)).thenReturn(houseIn);
        when(houseRepository.saveAndFlush(any(House.class))).thenAnswer(invocation -> invocation.getArgument(0));

        var result = houseService.switchHousePower(houseId);

        assertTrue(result.isEnabled());
        verify(houseRepository, times(1)).getReferenceById(houseId);
        verify(houseRepository, times(1)).saveAndFlush(houseIn);
    }

    @Test
    public void testGetHousesByOwnerIdEmpty() {
        var ownerId = "ownerId";
        List<House> houses = new ArrayList<>();

        when(houseRepository.findHousesByOwnerId(ownerId)).thenReturn(houses);

        var result = houseService.getHousesByOwnerId(ownerId);

        assertEquals(houses, result);
        verify(houseRepository, times(1)).findHousesByOwnerId(ownerId);
    }

    @Test
    public void testGetHousesByOwnerIdNonEmpty() {
        var ownerId = "ownerId";
        List<House> houses = new ArrayList<>();
        houses.add(new House());

        when(houseRepository.findHousesByOwnerId(ownerId)).thenReturn(houses);

        var result = houseService.getHousesByOwnerId(ownerId);

        assertEquals(houses, result);
        verify(houseRepository, times(1)).findHousesByOwnerId(ownerId);
    }

    @Test
    void createHouseTest() {
        NewHouse newHouse = new NewHouse()
                .name("name")
                .address("address")
                .ownerId("owner");

        when(houseRepository.saveAndFlush(any(House.class))).thenAnswer(invocation -> invocation.getArgument(0));

        House result = houseService.createHouse(newHouse);

        assertEquals(newHouse.getName(), result.getName());
        assertEquals(newHouse.getAddress(), result.getAddress());
        assertEquals(newHouse.getOwnerId(), result.getOwnerId());
        assertTrue(result.isEnabled());

        verify(houseRepository).saveAndFlush(result);
    }
    @Test
    public void testGetHouseById() {
        var id = "id";
        var response = new House()
                .setId(id)
                .setAddress("address")
                .setOwnerId("owner")
                .setName("name")
                .setEnabled(true);


        when(houseRepository.getReferenceById(id)).thenReturn(response);
        var result = houseService.getHouseById(id);

        assertEquals(response, result);
        verify(houseRepository, times(1)).getReferenceById(id);
    }

    @Test
    public void testUpdateHouseFull() {
        var id = "id";
        var updateHouse = new UpdateHouse()
                .address("address")
                .name("name");
        var modifiedHouse = new House()
                .setId(id)
                .setOwnerId("owner")
                .setEnabled(true);
        var expected = new House()
                .setId(id)
                .setAddress("address")
                .setOwnerId("owner")
                .setName("name")
                .setEnabled(true);

        when(houseRepository.saveAndFlush(modifiedHouse)).thenReturn(modifiedHouse);
        when(houseRepository.findById(id)).thenReturn(Optional.of(modifiedHouse));
        var result = houseService.updateHouse(id, updateHouse);

        assertEquals(expected.getName(), result.getName());
        assertEquals(expected.getAddress(), result.getAddress());
        assertEquals(expected.getId(), result.getId());
        assertEquals(expected.getOwnerId(), result.getOwnerId());
        assertEquals(expected.isEnabled(), result.isEnabled());
        verify(houseRepository, times(1)).saveAndFlush(modifiedHouse);
        verify(houseRepository, times(1)).findById(id);
    }

    @Test
    public void testUpdateHouseEmpty() {
        var id = "id";
        var updateHouse = new UpdateHouse();
        var modifiedHouse = new House()
                .setId(id)
                .setOwnerId("owner")
                .setEnabled(true);
        var expected = new House()
                .setId(id)
                .setOwnerId("owner")
                .setEnabled(true);

        when(houseRepository.saveAndFlush(modifiedHouse)).thenReturn(modifiedHouse);
        when(houseRepository.findById(id)).thenReturn(Optional.of(modifiedHouse));
        var result = houseService.updateHouse(id, updateHouse);

        assertEquals(expected.getName(), result.getName());
        assertEquals(expected.getAddress(), result.getAddress());
        assertEquals(expected.getId(), result.getId());
        assertEquals(expected.getOwnerId(), result.getOwnerId());
        assertEquals(expected.isEnabled(), result.isEnabled());
        verify(houseRepository, times(1)).saveAndFlush(modifiedHouse);
        verify(houseRepository, times(1)).findById(id);
    }

    @Test
    public void testUpdateHouseOnlyAddress() {
        var id = "id";
        var updateHouse = new UpdateHouse()
                .address("address");
        var modifiedHouse = new House()
                .setId(id)
                .setOwnerId("owner")
                .setEnabled(true);
        var expected = new House()
                .setId(id)
                .setOwnerId("owner")
                .setAddress("address")
                .setEnabled(true);

        when(houseRepository.saveAndFlush(modifiedHouse)).thenReturn(modifiedHouse);
        when(houseRepository.findById(id)).thenReturn(Optional.of(modifiedHouse));
        var result = houseService.updateHouse(id, updateHouse);

        assertEquals(expected.getName(), result.getName());
        assertEquals(expected.getAddress(), result.getAddress());
        assertEquals(expected.getId(), result.getId());
        assertEquals(expected.getOwnerId(), result.getOwnerId());
        assertEquals(expected.isEnabled(), result.isEnabled());
        verify(houseRepository, times(1)).saveAndFlush(modifiedHouse);
        verify(houseRepository, times(1)).findById(id);
    }

    @Test
    public void testUpdateHouseOnlyName() {
        var id = "id";
        var updateHouse = new UpdateHouse()
                .name("name");
        var modifiedHouse = new House()
                .setId(id)
                .setOwnerId("owner")
                .setEnabled(true);
        var expected = new House()
                .setId(id)
                .setOwnerId("owner")
                .setName("name")
                .setEnabled(true);

        when(houseRepository.saveAndFlush(modifiedHouse)).thenReturn(modifiedHouse);
        when(houseRepository.findById(id)).thenReturn(Optional.of(modifiedHouse));
        var result = houseService.updateHouse(id, updateHouse);

        assertEquals(expected.getName(), result.getName());
        assertEquals(expected.getAddress(), result.getAddress());
        assertEquals(expected.getId(), result.getId());
        assertEquals(expected.getOwnerId(), result.getOwnerId());
        assertEquals(expected.isEnabled(), result.isEnabled());
        verify(houseRepository, times(1)).saveAndFlush(modifiedHouse);
        verify(houseRepository, times(1)).findById(id);
    }

    @Test
    public void testDeleteHouse() {
        var id = "id";
        var response = new House()
                .setId(id)
                .setAddress("address")
                .setOwnerId("owner")
                .setName("name")
                .setEnabled(true);

        when(houseRepository.findById(id)).thenReturn(Optional.of(response));

        houseService.deleteHouse(id);

        verify(houseRepository, times(1)).deleteById(id);
    }
}
