package cz.muni.pa165.generated.power_control.PowerControlServerTests;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;
import jakarta.transaction.Transactional;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.web.servlet.MockMvc;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
@ExtendWith(MockitoExtension.class)
@Transactional
@Rollback
class PowerControlServerIntegrationTests {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @Test
    void createHouseTest() throws Exception {
        NewHouse newHouse = new NewHouse()
                .ownerId("owner")
                .address("address")
                .name("name");

        String jsonNewHouse = objectMapper.writeValueAsString(newHouse);

        var response = mockMvc.perform(post("/api/house")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewHouse))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString();
        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"enabled\":true"));
        assertTrue(response.contains("\"address\":\"address\""));
        assertTrue(response.contains("\"ownerId\":\"owner\""));
    }

    @Test
    void deleteExistingHouseTest() throws Exception {
        NewHouse newHouse = new NewHouse()
                .ownerId("owner")
                .address("address")
                .name("name");

        String jsonNewHouse = objectMapper.writeValueAsString(newHouse);

        var id = mockMvc.perform(post("/api/house")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewHouse))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        var response = mockMvc.perform(delete("/api/house")
                        .header("accept", "application/json")
                        .queryParam("houseId", id))
                .andExpect(status().isNoContent())
                .andReturn().getResponse().getContentAsString();
        assertEquals(response, "");
    }

    @Test
    void deleteNonExistingHouseTest() throws Exception {
        var response = mockMvc.perform(delete("/api/house")
                        .header("accept", "application/json")
                        .queryParam("houseId", "-1"))
                .andExpect(status().isNotFound())
                .andReturn().getResponse().getContentAsString();

        assertEquals(response, "");
    }

    @Test
    void getExistingHouseTest() throws Exception {
        NewHouse newHouse = new NewHouse()
                .ownerId("owner")
                .address("address")
                .name("name");

        String jsonNewHouse = objectMapper.writeValueAsString(newHouse);

        var id = mockMvc.perform(post("/api/house")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewHouse))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        var response = mockMvc.perform(get("/api/house")
                        .header("accept", "application/json")
                        .queryParam("id", id))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();
        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"enabled\":true"));
        assertTrue(response.contains("\"address\":\"address\""));
        assertTrue(response.contains("\"ownerId\":\"owner\""));
    }

    @Test
    void getNonExistingHouseTest() throws Exception {
        var response = mockMvc.perform(get("/api/house")
                        .header("accept", "application/json")
                        .queryParam("id", "-1"))
                .andExpect(status().isNotFound())
                .andReturn().getResponse().getContentAsString();
        assertEquals(response, "");
    }

    @Test
    void updateExistingHouseTest() throws Exception {
        NewHouse newHouse = new NewHouse()
                .ownerId("owner")
                .address("address")
                .name("name");

        String jsonNewHouse = objectMapper.writeValueAsString(newHouse);

        var id = mockMvc.perform(post("/api/house")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewHouse))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        var updatedHouse = new UpdateHouse()
                .address("newAddress")
                .name("newName");

        String jsonUpdatedHouse = objectMapper.writeValueAsString(updatedHouse);

        var response = mockMvc.perform(patch("/api/house")
                        .header("accept", "application/json")
                        .queryParam("houseId", id)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonUpdatedHouse))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();
        assertTrue(response.contains("\"name\":\"newName\""));
        assertTrue(response.contains("\"enabled\":true"));
        assertTrue(response.contains("\"address\":\"newAddress\""));
        assertTrue(response.contains("\"ownerId\":\"owner\""));
    }

    @Test
    void updateNonExistingHouseTest() throws Exception {
        var updatedHouse = new UpdateHouse()
                .address("newAddress")
                .name("newName");

        String jsonUpdatedHouse = objectMapper.writeValueAsString(updatedHouse);

        var response = mockMvc.perform(patch("/api/house")
                        .header("accept", "application/json")
                        .queryParam("houseId", "-1")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonUpdatedHouse))
                .andExpect(status().isNotFound())
                .andReturn().getResponse().getContentAsString();
        assertEquals(response, "");
    }

    @Test
    void switchPowerInExistingHouseTest() throws Exception {
        NewHouse newHouse = new NewHouse()
                .ownerId("owner")
                .address("address")
                .name("name");

        String jsonNewHouse = objectMapper.writeValueAsString(newHouse);

        var id = mockMvc.perform(post("/api/house")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewHouse))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString()
                .substring(7, 39);

        var response = mockMvc.perform(patch("/api/houses/switchPower")
                        .header("accept", "application/json")
                        .queryParam("id", id))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();
        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"enabled\":false"));
        assertTrue(response.contains("\"address\":\"address\""));
        assertTrue(response.contains("\"ownerId\":\"owner\""));
    }

    @Test
    void switchPowerInNonExistingHouseTest() throws Exception {
        var response = mockMvc.perform(patch("/api/houses/switchPower")
                        .header("accept", "application/json")
                        .queryParam("id", "-1")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andReturn().getResponse().getContentAsString();
        assertEquals(response, "");
    }

    @Test
    void getHousesByOwnerTest() throws Exception {
        NewHouse newHouse = new NewHouse()
                .ownerId("owner")
                .address("address")
                .name("name");

        String jsonNewHouse = objectMapper.writeValueAsString(newHouse);

        mockMvc.perform(post("/api/house")
                        .header("accept", "application/json")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonNewHouse))
                .andExpect(status().isCreated())
                .andReturn().getResponse().getContentAsString();

        var response = mockMvc.perform(get("/api/houses")
                        .header("accept", "application/json")
                        .queryParam("ownerId", "owner"))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();
        assertTrue(response.contains("\"name\":\"name\""));
        assertTrue(response.contains("\"enabled\":true"));
        assertTrue(response.contains("\"address\":\"address\""));
        assertTrue(response.contains("\"ownerId\":\"owner\""));
    }

    @Test
    void getNonExistingHousesByOwnerTest() throws Exception {
        var response = mockMvc.perform(get("/api/houses")
                        .header("accept", "application/json")
                        .queryParam("ownerId", "owner")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andReturn().getResponse().getContentAsString();
        assertEquals(response, "{\"houses\":[]}");
    }
}