package cz.muni.pa165.generated.power_control.PowerControlServerTests;

import cz.muni.pa165.generated.power_control.PowerControlServer;
import cz.muni.pa165.generated.power_control.facade.HouseFacade;
import cz.muni.pa165.generated.power_control.model.HouseResponse;
import cz.muni.pa165.generated.power_control.model.HouseResponseList;
import cz.muni.pa165.generated.power_control.model.NewHouse;
import cz.muni.pa165.generated.power_control.model.UpdateHouse;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.simple.SimpleMeterRegistry;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class PowerControlServerUnitTests {

    private PowerControlServer powerControlServer;

    @Mock
    private HouseFacade houseFacade;

    private MeterRegistry meterRegistry;

    @BeforeEach
    public void setUp() {
        meterRegistry = new SimpleMeterRegistry();
        powerControlServer = new PowerControlServer(houseFacade, meterRegistry);
    }

    @Test
    public void testSwitchPowerInHouse() {
        var id = "houseId";
        var enabledHouse = new HouseResponse();
        enabledHouse.setEnabled(true);

        when(houseFacade.switchPower(id)).thenReturn(enabledHouse);

        ResponseEntity<HouseResponse> response = powerControlServer.switchPowerInHouse(id);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(enabledHouse, response.getBody());
        verify(houseFacade, times(1)).switchPower(id);
    }

    @Test
    public void testGetHousesByOwnerEmpty() {
        var ownerId = "owner";
        var houseResponseList = new HouseResponseList();

        when(houseFacade.getAllByOwnerId(ownerId)).thenReturn(houseResponseList);

        ResponseEntity<HouseResponseList> response = powerControlServer.getHousesByOwner(ownerId);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(houseResponseList, response.getBody());
        verify(houseFacade, times(1)).getAllByOwnerId(ownerId);
    }

    @Test
    public void testGetHousesByOwnerSingle() {
        var ownerId = "owner";
        var house = new HouseResponse().ownerId(ownerId);
        var houseResponseList = new HouseResponseList();
        houseResponseList.addHousesItem(house);

        when(houseFacade.getAllByOwnerId(ownerId)).thenReturn(houseResponseList);

        ResponseEntity<HouseResponseList> response = powerControlServer.getHousesByOwner(ownerId);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(houseResponseList, response.getBody());
        verify(houseFacade, times(1)).getAllByOwnerId(ownerId);
    }

    @Test
    public void testGetHousesByOwnerMultiple() {
        var ownerId = "owner";
        var house = new HouseResponse().ownerId(ownerId);
        var house2 = new HouseResponse().ownerId(ownerId);
        var houseResponseList = new HouseResponseList();
        houseResponseList.addHousesItem(house);
        houseResponseList.addHousesItem(house2);

        when(houseFacade.getAllByOwnerId(ownerId)).thenReturn(houseResponseList);

        ResponseEntity<HouseResponseList> response = powerControlServer.getHousesByOwner(ownerId);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(houseResponseList, response.getBody());
        verify(houseFacade, times(1)).getAllByOwnerId(ownerId);
    }

    @Test
    public void testCreateHouse() {
        var newHouse = new NewHouse()
                .ownerId("owner")
                .name("house")
                .address("address");

        var houseResponse = new HouseResponse()
                .id("id")
                .ownerId("owner")
                .name("house")
                .address("address");

        when(houseFacade.create(newHouse)).thenReturn(houseResponse);

        ResponseEntity<HouseResponse> response = powerControlServer.create(newHouse);

        assertEquals(HttpStatus.CREATED, response.getStatusCode());
        assertEquals(houseResponse, response.getBody());
        verify(houseFacade, times(1)).create(newHouse);
    }

    @Test
    public void testGetHouse() {
        var houseId = "id";
        var houseResponse = new HouseResponse()
                .id(houseId)
                .ownerId("owner")
                .name("house")
                .address("address");

        when(houseFacade.get(houseId)).thenReturn(houseResponse);

        ResponseEntity<HouseResponse> response = powerControlServer.get(houseId);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(houseResponse, response.getBody());
        verify(houseFacade, times(1)).get(houseId);
    }

    @Test
    public void testUpdateHouse() {
        var houseId = "id";
        var updateHouse = new UpdateHouse()
                .name("house")
                .address("address");

        var houseResponse = new HouseResponse()
                .id(houseId)
                .name("house")
                .address("address");

        when(houseFacade.update(houseId, updateHouse)).thenReturn(houseResponse);

        ResponseEntity<HouseResponse> response = powerControlServer.update(houseId, updateHouse);

        assertEquals(HttpStatus.OK, response.getStatusCode());
        assertEquals(houseResponse, response.getBody());
        verify(houseFacade, times(1)).update(houseId, updateHouse);
    }

    @Test
    public void testDeleteHouse() {
        var houseId = "id";

        powerControlServer.delete(houseId);
        verify(houseFacade, times(1)).delete(houseId);
    }
}

//package cz.muni.pa165.generated.power_control;
//
//import cz.muni.pa165.generated.power_control.data.repository.HouseRepository;
//import cz.muni.pa165.generated.power_control.model.HouseResponse;
//import cz.muni.pa165.generated.power_control.model.NewHouse;
//import cz.muni.pa165.generated.power_control.service.HouseService;
//import org.junit.jupiter.api.Assertions;
//import org.junit.jupiter.api.Test;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
//import org.springframework.boot.test.context.SpringBootTest;
//import org.springframework.boot.test.mock.mockito.MockBean;
//import org.springframework.http.MediaType;
//import org.springframework.test.web.servlet.MockMvc;
//import org.springframework.test.web.servlet.MvcResult;
//
//import static org.mockito.BDDMockito.given;
//import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
//import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
//
//@SpringBootTest
//@AutoConfigureMockMvc
//class PowerControlServerTests {
//    private static final Logger log = LoggerFactory.getLogger(PowerControlServerTests.class);
//
//    @Autowired
//    private MockMvc mockMvc;
//
//    @MockBean
//    private HouseRepository repository;
//
//    static HouseResponse storedHouse = new HouseResponse();
//
//    @Test
//    void createNewHouse() throws Exception {
//        NewHouse newHouse = new NewHouse()
//                .name();
//        given(mockHouseService.createHouse(newHouse)).willReturn(storedHouse);
//        mockMvc.perform(post("/api/house?userId=1")
//                        .header("accept", "application/json")
//                        .contentType(MediaType.APPLICATION_JSON))
//                .andExpect(status().isCreated());
//    }
//}
//

//
//@SpringBootTest
//@AutoConfigureMockMvc
//class PowerControlServerTests {
//
//    private static final Logger log = LoggerFactory.getLogger(PowerControlServerTests.class);
//
//    @Autowired
//    private MockMvc mockMvc;
//
//    @MockBean
//    private HouseService mockHouseService;
//
//    static HouseResponse storedHouse = new HouseResponse();
//
//    @Test
//    void createNewHouse() throws Exception {
//        NewHouse newHouse = new NewHouse();
//        given(mockHouseService.createHouse(newHouse)).willReturn(storedHouse);
//        mockMvc.perform(post("/api/house?userId=1")
//                        .header("accept", "application/json")
//                        .contentType(MediaType.APPLICATION_JSON))
//                .andExpect(status().isCreated());
//    }
//
//    @Test
//    void getExistingHouse() throws Exception {
//        log.debug("getHouseById existing");
//        given(mockHouseService.getHouseById("0")).willReturn(storedHouse);
//        mockMvc.perform(get("/api/house?houseId=0"))
//                .andExpect(status().isOk());
//    }
//
//    @Test
//    void getNonExistingHouse() throws Exception {
//        log.debug("getHouseById non-existing");
//        given(mockHouseService.getHouseById("-1")).willReturn(null);
//        MvcResult result = mockMvc.perform(get("/api/house?houseId=-1"))
//                .andExpect(status().isOk())
//                .andReturn();
//        String actualJson = result.getResponse().getContentAsString();
//        Assertions.assertEquals("", actualJson);
//    }
//
//    @Test
//    void switchHousePower() throws Exception {
//        log.debug("switch house power");
//        given(mockHouseService.switchHousePower("0")).willReturn(storedHouse);
//        MvcResult result = mockMvc.perform(patch("/api/house?id=0"))
//                .andExpect(status().isOk())
//                .andReturn();
//        String actualJson = result.getResponse().getContentAsString();
//        Assertions.assertEquals("{\"id\":null,\"powerEnabled\":null,\"address\":null}", actualJson);
//    }
//
//    @Test
//    void deleteHouse() throws Exception {
//        log.debug("delete house");
//        MvcResult result = mockMvc.perform(delete("/api/house?houseId=0"))
//                .andExpect(status().isNoContent())
//                .andReturn();
//        String actualJson = result.getResponse().getContentAsString();
//        Assertions.assertEquals("", actualJson);
//    }
//
//}
