package cz.muni.pa165.generated.user_management;

import cz.muni.pa165.generated.user_management.api.UserApiDelegate;
import cz.muni.pa165.generated.user_management.facades.UserFacade;
import cz.muni.pa165.generated.user_management.model.UserRequest;
import cz.muni.pa165.generated.user_management.model.UserResponse;
import io.micrometer.core.instrument.Counter;
import io.micrometer.core.instrument.MeterRegistry;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.TransactionSystemException;

import java.util.List;

@Controller
public class UserServer implements UserApiDelegate {
    private final UserFacade userFacade;
    private final MeterRegistry meterRegistry;
    private final Counter requestCounter;

    public UserServer(UserFacade userFacade, MeterRegistry meterRegistry) {
        this.userFacade = userFacade;
        this.meterRegistry = meterRegistry;
        this.requestCounter = meterRegistry.counter("UserManagement.requestCounter");
    }

    @Override
    public ResponseEntity<List<UserResponse>> getUsers() {
        requestCounter.increment();
        return new ResponseEntity<>(userFacade.getUsers(), HttpStatus.OK);
    }

    @Override
    public ResponseEntity<UserResponse> getUser(String userId) {
        try {
            requestCounter.increment();
            UserResponse userResponse = userFacade.getUser(userId);
            return new ResponseEntity<>(userResponse, userResponse == null ? HttpStatus.NOT_FOUND : HttpStatus.OK);
        } catch (IllegalArgumentException e) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<UserResponse> postUserCreate(UserRequest userRequest) {
        try {
            requestCounter.increment();
            return new ResponseEntity<>(userFacade.postUserCreate(userRequest), HttpStatus.CREATED);
        } catch (TransactionSystemException e) {
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);
        }
    }

    @Override
    public ResponseEntity<Void> deleteUser(String userId) {
        try {
            requestCounter.increment();
            boolean deleted = userFacade.deleteUser(userId);
            return new ResponseEntity<>(null, deleted ? HttpStatus.NO_CONTENT : HttpStatus.NOT_FOUND);
        } catch (IllegalArgumentException e) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }

    @Override
    public ResponseEntity<UserResponse> updateUser(String userId, UserRequest userRequest) {
        try {
            requestCounter.increment();
            UserResponse userResponse = userFacade.updateUser(userId, userRequest);
            return new ResponseEntity<>(userResponse, userResponse == null ? HttpStatus.NOT_FOUND : HttpStatus.OK);
        } catch (IllegalArgumentException e) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
    }
}
