package cz.muni.pa165.generated.user_management.facades;

import cz.muni.pa165.generated.user_management.mappers.UserMapper;
import cz.muni.pa165.generated.user_management.model.UserRequest;
import cz.muni.pa165.generated.user_management.model.UserResponse;
import cz.muni.pa165.generated.user_management.models.User;
import cz.muni.pa165.generated.user_management.services.UserService;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
@Transactional
public class UserFacade {
    private final UserMapper userMapper;
    private final UserService userService;

    public UserFacade(UserMapper userMapper, UserService userService) {
        this.userMapper = userMapper;
        this.userService = userService;
    }

    public List<UserResponse> getUsers() {
        return userService.getUsers().stream().map(userMapper::mapToDto).toList();
    }

    public UserResponse getUser(String userId) {
        Optional<User> userOptional = userService.getUser(userId);
        return userOptional.map(userMapper::mapToDto).orElse(null);
    }

    public UserResponse postUserCreate(UserRequest userRequest) {
        User user = userService.postUserCreate(userRequest);
        return userMapper.mapToDto(user);
    }

    public boolean deleteUser(String userId) {
        return userService.deleteUser(userId);
    }

    public UserResponse updateUser(String userId, UserRequest userRequest) {
        User user = userService.updateUser(userId, userRequest);
        if (user == null) {
            return null;
        }

        return userMapper.mapToDto(user);
    }
}
