package cz.muni.pa165.generated.user_management.services;

import cz.muni.pa165.generated.user_management.model.UserAuth;
import cz.muni.pa165.generated.user_management.model.UserRequest;
import cz.muni.pa165.generated.user_management.models.User;
import cz.muni.pa165.generated.user_management.repositories.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Service
public class UserService {
    private static final Logger log = LoggerFactory.getLogger(UserService.class);
    private final UserRepository userRepository;
    private final PasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public UserService(UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    public List<User> getUsers() {
        return userRepository.findAll();
    }

    public Optional<User> getUser(String userId) {
        Optional<User> user = userRepository.findById(UUID.fromString(userId));
        log.debug("requested user {}", user);
        return user;
    }

    public User postUserCreate(UserRequest userRequest) {
        User user = new User(
                userRequest.getName(),
                userRequest.getEmail(),
                passwordEncoder.encode(userRequest.getPassword())
        );
        userRepository.save(user);

        log.debug("created user {}", user);
        return user;
    }

    public boolean deleteUser(String userId) {
        var userOptional = userRepository.findById(UUID.fromString(userId));
        if (userOptional.isPresent()) {
            userRepository.deleteById(UUID.fromString(userId));
            log.debug("deleted user {}", userOptional.get());
            return true;
        }

        log.debug("failed to delete user with id {}", userId);
        return false;
    }

    public User updateUser(String userId, UserRequest userRequest) {
        var userOptional = userRepository.findById(UUID.fromString(userId));
        if (userOptional.isEmpty()) {
            log.debug("failed to update user with id {}", userId);
            return null;
        }

        var user = userOptional.get();
        user.setName(userRequest.getName());
        user.setEmail(userRequest.getEmail());
        user.setPassword(userRequest.getPassword());
        userRepository.save(user);

        return user;
    }
}
