package cz.muni.pa165.generated.user_management;

import cz.muni.pa165.generated.user_management.model.UserRequest;
import cz.muni.pa165.generated.user_management.models.User;
import cz.muni.pa165.generated.user_management.repositories.UserRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.UUID;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
@ExtendWith(MockitoExtension.class)
public class UserServerTests {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private UserRepository userRepository;

    private User user1;
    private String uuid1;

    @BeforeEach
    public void setUp() {
        userRepository.deleteAll();

        user1 = new User("John Doe", "john.doe@example.com", "password1");
        user1 = userRepository.save(user1);
        uuid1 = user1.getId().toString();
    }

    @Test
    public void getUsersTest() throws Exception {
        mockMvc.perform(get("/api/user"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$[0].id").value(uuid1))
                .andExpect(jsonPath("$[0].name").value(user1.getName()))
                .andExpect(jsonPath("$[0].email").value(user1.getEmail()));
    }

    @Test
    public void getUserTest() throws Exception {
        mockMvc.perform(get("/api/user/{userId}", uuid1))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id").value(uuid1))
                .andExpect(jsonPath("$.name").value(user1.getName()))
                .andExpect(jsonPath("$.email").value(user1.getEmail()));
    }

    @Test
    public void getUserNotFoundTest() throws Exception {
        String nonExistentUuid = UUID.randomUUID().toString();
        mockMvc.perform(get("/users/{userId}", nonExistentUuid))
                .andExpect(status().isNotFound());
    }

    @Test
    public void postUserCreateTest() throws Exception {
        UserRequest userRequest = new UserRequest();
        userRequest.setName("Jane Doe");
        userRequest.setEmail("jane.doe@example.com");
        userRequest.setPassword("password2");

        mockMvc.perform(post("/api/user")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(JsonUtil.toJson(userRequest)))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.name").value(userRequest.getName()))
                .andExpect(jsonPath("$.email").value(userRequest.getEmail()));
    }

    @Test
    public void deleteUserTest() throws Exception {
        mockMvc.perform(delete("/api/user/{userId}", uuid1))
                .andExpect(status().isNoContent());

        mockMvc.perform(get("/users/{userId}", uuid1))
                .andExpect(status().isNotFound());
    }

    @Test
    public void deleteUserNotFoundTest() throws Exception {
        String nonExistentUuid = UUID.randomUUID().toString();
        mockMvc.perform(delete("/users/{userId}", nonExistentUuid))
                .andExpect(status().isNotFound());
    }

    @Test
    public void updateUserTest() throws Exception {
        UserRequest userRequest = new UserRequest();
        userRequest.setName("John Updated");
        userRequest.setEmail("john.updated@example.com");
        userRequest.setPassword("password3");

        mockMvc.perform(patch("/api/user/{userId}", uuid1)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(JsonUtil.toJson(userRequest)))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id").value(uuid1))
                .andExpect(jsonPath("$.name").value(userRequest.getName()))
                .andExpect(jsonPath("$.email").value(userRequest.getEmail()));
    }

    @Test
    public void updateUserNotFoundTest() throws Exception {
        UserRequest userRequest = new UserRequest();
        userRequest.setName("John Updated");
        userRequest.setEmail("john.updated@example.com");
        userRequest.setPassword("password3");

        String nonExistentUuid = UUID.randomUUID().toString();
        mockMvc.perform(patch("/users/{userId}", nonExistentUuid)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(JsonUtil.toJson(userRequest)))
                .andExpect(status().isNotFound());
    }
}