package cz.muni.pa165.generated.user_management.unit_tests;

import cz.muni.pa165.generated.user_management.facades.UserFacade;
import cz.muni.pa165.generated.user_management.mappers.UserMapper;
import cz.muni.pa165.generated.user_management.model.UserRequest;
import cz.muni.pa165.generated.user_management.model.UserResponse;
import cz.muni.pa165.generated.user_management.models.User;
import cz.muni.pa165.generated.user_management.services.UserService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class UserFacadeTests {

    @InjectMocks
    private UserFacade userFacade;

    @Mock
    private UserService userService;

    @Mock
    private UserMapper userMapper;

    private User user1;
    private User user2;
    private UserResponse userResponse1;
    private UserResponse userResponse2;
    private String uuid1;
    private String uuid2;

    @BeforeEach
    public void setUp() {
        uuid1 = UUID.randomUUID().toString();
        uuid2 = UUID.randomUUID().toString();

        user1 = new User("John Doe", "john.doe@example.com", "password1");
        user2 = new User("Jane Doe2", "jane.doe2@example.com", "password2");

        userResponse1 = new UserResponse();
        userResponse1.setId(uuid1);
        userResponse1.setName(user1.getName());
        userResponse1.setEmail(user1.getEmail());

        userResponse2 = new UserResponse();
        userResponse2.setId(uuid2);
        userResponse2.setName(user2.getName());
        userResponse2.setEmail(user2.getEmail());
    }

    @Test
    public void getUsersTest() {
        when(userService.getUsers()).thenReturn(Arrays.asList(user1, user2));
        when(userMapper.mapToDto(user1)).thenReturn(userResponse1);
        when(userMapper.mapToDto(user2)).thenReturn(userResponse2);

        List<UserResponse> users = userFacade.getUsers();

        assertNotNull(users);
        assertEquals(2, users.size());
        assertEquals(userResponse1, users.get(0));
        assertEquals(userResponse2, users.get(1));
    }

    @Test
    public void getUserTest() {
        when(userService.getUser(uuid1)).thenReturn(Optional.of(user1));
        when(userMapper.mapToDto(user1)).thenReturn(userResponse1);

        UserResponse userResponse = userFacade.getUser(uuid1);

        assertNotNull(userResponse);
        assertEquals(userResponse1, userResponse);
    }

    @Test
    public void getUserNotFoundTest() {
        when(userService.getUser(uuid1)).thenReturn(Optional.empty());

        UserResponse userResponse = userFacade.getUser(uuid1);

        assertNull(userResponse);
    }

    @Test
    public void postUserCreateTest() {
        UserRequest userRequest = new UserRequest();
        userRequest.setName(user1.getName());
        userRequest.setEmail(user1.getEmail());
        userRequest.setPassword(user1.getPassword());

        when(userService.postUserCreate(userRequest)).thenReturn(user1);
        when(userMapper.mapToDto(user1)).thenReturn(userResponse1);

        UserResponse createdUserResponse = userFacade.postUserCreate(userRequest);

        assertNotNull(createdUserResponse);
        assertEquals(userResponse1, createdUserResponse);
    }


    @Test
    public void deleteUserTest() {
        when(userService.deleteUser(uuid1)).thenReturn(true);

        boolean deleted = userFacade.deleteUser(uuid1);

        assertTrue(deleted);
    }

    @Test
    public void deleteUserNotFoundTest() {
        when(userService.deleteUser(uuid1)).thenReturn(false);

        boolean deleted = userFacade.deleteUser(uuid1);

        assertFalse(deleted);
    }

    @Test
    public void updateUserTest() {
        UserRequest userRequest = new UserRequest();
        userRequest.setName(user2.getName());
        userRequest.setEmail(user2.getEmail());
        userRequest.setPassword(user2.getPassword());

        when(userService.updateUser(uuid1, userRequest)).thenReturn(user2);
        when(userMapper.mapToDto(user2)).thenReturn(userResponse2);

        UserResponse updatedUserResponse = userFacade.updateUser(uuid1, userRequest);

        assertNotNull(updatedUserResponse);
        assertEquals(userResponse2, updatedUserResponse);
    }

    @Test
    public void updateUserNotFoundTest() {
        UserRequest userRequest = new UserRequest();
        userRequest.setName(user2.getName());
        userRequest.setEmail(user2.getEmail());
        userRequest.setPassword(user2.getPassword());

        when(userService.updateUser(uuid1, userRequest)).thenReturn(null);

        UserResponse updatedUserResponse = userFacade.updateUser(uuid1, userRequest);

        assertNull(updatedUserResponse);
    }
}