package cz.muni.pa165.generated.user_management.unit_tests;


import cz.muni.pa165.generated.user_management.model.UserRequest;
import cz.muni.pa165.generated.user_management.models.User;
import cz.muni.pa165.generated.user_management.repositories.UserRepository;
import cz.muni.pa165.generated.user_management.services.UserService;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;


@ExtendWith(MockitoExtension.class)
public class UserServiceTests {

    @InjectMocks
    private UserService userService;

    @Mock
    private UserRepository userRepository;

    @Test
    public void testGetUser() {
        var id = UUID.randomUUID();
        var stringId = id.toString();
        var response = new User()
                .setId(id)
                .setEmail("testEmail")
                .setName("testName")
                .setPassword("testPassword");

        when(userRepository.findById(id)).thenReturn(Optional.ofNullable(response));
        var result = userService.getUser(stringId);

        assertTrue(result.isPresent());
        assertEquals(response, result.get());
        verify(userRepository, times(1)).findById(id);
    }

    @Test
    public void testGetUsersEmpty() {
        List<User> users = new ArrayList<>();

        when(userRepository.findAll()).thenReturn(users);

        var result = userService.getUsers();
        assertEquals(users, result);
        verify(userRepository, times(1)).findAll();
    }

    @Test
    public void testGetUsersNotEmpty() {
        List<User> users = new ArrayList<>();
        users.add(new User());

        when(userRepository.findAll()).thenReturn(users);

        var result = userService.getUsers();
        assertEquals(users, result);
        verify(userRepository, times(1)).findAll();
    }

    @Test
    public void testPostUserCreate() {
        UserRequest userRequest = new UserRequest();
        userRequest.setEmail("testEmail");
        userRequest.setName("testName");
        userRequest.setPassword("testPassword");

        when(userRepository.save(any(User.class))).thenAnswer(invocation -> invocation.getArgument(0));

        User result = userService.postUserCreate(userRequest);

        assertEquals(result.getName(), userRequest.getName());
        assertEquals(result.getEmail(), userRequest.getEmail());

        verify(userRepository, times(1)).save(any(User.class));
    }

    @Test
    public void testDeleteUserSuccess() {
        var id = UUID.randomUUID();
        var stringId = id.toString();
        var user = new User()
                .setId(id);

        when(userRepository.findById(any(UUID.class))).thenAnswer(invocation -> Optional.ofNullable(user));

        var result = userService.deleteUser(stringId);
        assertTrue(result);

        verify(userRepository, times(1)).findById(id);
        verify(userRepository, times(1)).deleteById(id);
    }

    @Test
    public void testDeleteUserFail() {
        var id = UUID.randomUUID();
        var stringId = id.toString();

        when(userRepository.findById(any(UUID.class))).thenAnswer(invocation -> Optional.empty());

        var result = userService.deleteUser(stringId);
        assertFalse(result);

        verify(userRepository, times(1)).findById(id);
        verify(userRepository, times(0)).deleteById(any(UUID.class));
    }

    @Test
    public void testUpdateUserFull() {
        var id = UUID.randomUUID();
        var stringId = id.toString();

        var userRequest = new UserRequest();
        userRequest.setName("testName");
        userRequest.setEmail("testEmail");
        userRequest.setPassword("testPassword");

        var expected = new User();
        expected.setId(id);
        expected.setName("testName");
        expected.setEmail("testEmail");
        expected.setPassword("testPassword");


        var originalUser = new User();
        originalUser.setId(id);

        when(userRepository.findById(any(UUID.class))).thenReturn(Optional.of(originalUser));
        when(userRepository.save(any(User.class))).then(invocation -> invocation.getArgument(0));

        var result = userService.updateUser(stringId, userRequest);

        assertEquals(expected.getId(), result.getId());
        assertEquals(expected.getName(), result.getName());
        assertEquals(expected.getEmail(), result.getEmail());
        assertEquals(expected.getPassword(), result.getPassword());
        verify(userRepository, times(1)).save(originalUser);
        verify(userRepository, times(1)).findById(id);
    }

    @Test
    public void testUpdateUserFail() {
        var id = UUID.randomUUID();
        var stringId = id.toString();

        var userRequest = new UserRequest();
        userRequest.setName("testName");
        userRequest.setEmail("testEmail");
        userRequest.setPassword("testPassword");


        var originalUser = new User();
        originalUser.setId(id);

        when(userRepository.findById(any(UUID.class))).thenReturn(Optional.empty());

        var result = userService.updateUser(stringId, userRequest);

        assertNull(result);
        verify(userRepository, times(1)).findById(id);
    }
}
